import java.util.*;
import java.io.*;


/**
 * class Daten.
 * Hier werden die Daten gespeichert und verwaltet.
 * @author Sascha Nemecek/9825815.
 * @version 1.0 15.01.1999
 */
class Daten extends Observable {
    Liste artikeldaten = new Liste ();
    Liste rechnungsdaten = new Liste ();
    Liste gruppendaten = new Liste ();
    
    /**
     * Konstruktor: Daten.
     * @param pfad Pfad der Daten.
     * @param artikelfile Name des Files, in dem die Artikeldaten gespeichert sind.
     * @param gruppenfile Name des Files, in dem die Gruppendaten gespeichert sind.
     * @exception MyException Wirft bei Fehler eine Meldung an das aufrufenden Programm.
     * @see MyException
     */
    public Daten (String pfad, String artikelfile, String gruppenfile) throws MyException {
	try {
	    ReadArtikeldaten (pfad, artikelfile);
	    ReadRechnungsdaten (pfad);
	    ReadGruppendaten (pfad, gruppenfile);

	    // Testausgabe der Daten
/*	    for (artikeldaten.gotoStart (); !artikeldaten.eol(); artikeldaten.gotoNext ())
		System.out.println (((Artikel)artikeldaten.getData()).artikel + " " + ((Artikel)artikeldaten.getData()).name + " " + ((Artikel)artikeldaten.getData()).preis);
		
	    for (gruppendaten.gotoStart (); !gruppendaten.eol(); gruppendaten.gotoNext ())
		System.out.println (((Gruppe)gruppendaten.getData()).artikel + " " + ((Gruppe)gruppendaten.getData()).name);
		
	    for (rechnungsdaten.gotoStart (); !rechnungsdaten.eol(); rechnungsdaten.gotoNext ())
		for (((Rechnung)rechnungsdaten.getData()).elements.gotoStart (); !((Rechnung)rechnungsdaten.getData()).elements.eol (); ((Rechnung)rechnungsdaten.getData()).elements.gotoNext ())
		    System.out.println (((Rechnung)rechnungsdaten.getData()).rechnungsNr + " " + ((Rechnung)rechnungsdaten.getData()).kassa + " " + ((RechnungsElement)((Rechnung)rechnungsdaten.getData()).elements.getData ()).artikel + " " + ((RechnungsElement)((Rechnung)rechnungsdaten.getData()).elements.getData ()).menge);
		    */
	    }
	catch (MyException e)
	    {throw new MyException (e.getMessage ());}
	    
	Observable o = new Observable ();
    	}
	
    
    /**
     * methode Changed ().
     * Benachrichtigt Observer.
     */
    public void Changed () {
	setChanged ();
	notifyObservers ();
	}


/********************************************************************************************
    Ab hier werden Artikeldaten behandelt
********************************************************************************************/


    /**
     *  Methode: ReadArtikeldaten.
     *  Liet das durch pfad und filename angegeben File ein und Wertet den Inhalt als Artikeldaten aus.
     *  Falls der Inhalt gltig ist, wird er in einer Liste abgelegt.
     *  @param String pfad: Gibt das Verzeichnis des einzulesenden Files an.
     *  @param String filename: Gibt den Filenamen des einzulesenden Files an.
     *  @exception MyException Wirft bei Fehler eine Meldung an aufrufende Methode.
     *  @see StringtoArtikel
     *	@see MyException
     */
   private void ReadArtikeldaten (String pfad, String filename) throws MyException {
	String s = null;
	ReadFile file;

	// Versucht File zu ffnen. Wenn nicht mglich, Abbruch
	file = new ReadFile (pfad, filename);

	// Liet solange Artikel ein, bis keine mehr vorhanden sind    
	while ((s = file.readLine()) != null) {
	    // Wenn Leerzeile, nchste Zeile einlesen
	    if (s.length () == 0) continue;
	    
	    try {
		// Wandelt einen String ins Artikelformat um
		StringtoArtikel (s);
		}
	    catch (MyException e) {
		// Fehlermeldung (ErrorDialog)
		System.out.println ("Fehler: " + e.getMessage());
		}
	    }
	    
	file.close ();
	if (artikeldaten.isEmpty())
	    throw new MyException ("Es wurden keine Daten eingelesen - Programmabbruch");
	}
    
    
    /**
     * Methode: String to Artikel.
     * Versucht den bergebenen String in der Liste "Artikeldaten" als "Artikel" abzulegen.
     * @param s - der umzuwandelnde String.
     * @exception MyException Wirft bei Fehler eine Meldung an aufrufende Methode.
     * @see Daten$Artikel
     */
    private void StringtoArtikel (String s) throws MyException
	{
	int nummer = 0;
	String name = null;
	double preis = 0;
	
	// Initialisiert Stringteiler und definiert die Trennzeichen
	StringTokenizer T = new StringTokenizer (s, ", ");

	try {
	    // berprft, ob die Parameteranzahl korrekt ist
	    if ((T.countTokens () != 3) || (s.charAt (s.length () - 1) == ','))
		throw new MyException ("Falsche Syntax: " + s + " - Datensatz ignoriert.");

	    // Liet die Artikelnummer, wenn nicht dreistellig Abbruch
	    name = T.nextToken ();
	    if (name.length() !=3)
		throw new MyException ("Die Artikelnummer " + nummer + " ist ungltig. Sie muss 3stellig sein. Datensatz ignoriert.");
	    nummer = Integer.parseInt (name);
	    
	    if ((nummer < 100) || (nummer > 399))
		throw new MyException ("Die Artikelnummer " + nummer +  " ist ungltig. Sie muss zw. 100 und 399 liegen. Datensatz ignoriert.");

	    // Liet Artikelnamen, berprft, ob der Artikelname zulssig ist, und bricht bei Fehler ab
	    name = T.nextToken ();
	    if (name.length() > 20)
		throw new MyException ("Artikelbezeichnung zu lang (max. 20 Zeichen). Datensatz ignoriert.");
    
	    // Liet den Preis
	    preis = Double.valueOf(T.nextToken()).doubleValue();
	    } 
	catch (NumberFormatException e) {
	    throw new MyException (e.getMessage());
	    }
	    
	// Wenn kein Fehler bei der Umwandlung aufgetreten ist, Datensatz speichern
	artikeldaten.add (new Artikel (nummer, name,preis));
	}
    
    
    /**
     * Methode getArtikelName ().
     * Liefert den zur Artikelnummer gehrigen Namen.
     * @param nr die Artikelnummer.
     * @return string mit dem Artikelnamen.
     */
    public String getArtikelName (int nr) {
	for (artikeldaten.gotoStart (); !artikeldaten.eol (); artikeldaten.gotoNext ())
	    if (((Artikel) artikeldaten.getData ()).artikel == nr)
		return ((Artikel) artikeldaten.getData ()).name;
	return null;
	}

		
/********************************************************************************************
    Ab hier werden die Rechnungsdaten behandelt
********************************************************************************************/


    /**
     * Methode: ReadRechnungsdaten.
     * Liet alle Rechnungsfiles aus dem angegebenen Verzeichnis ein.
     * @param String pfad: Verzeichnis aus dem die Rechnungsfiles gelesen werden sollen.
     * @exception MyException Wirft bei Fehler eine Meldung an aufrufende Methode.
     * @see MyException
     */
    public void ReadRechnungsdaten (String pfad) throws MyException {
	// ffnet Directory
	File file = new File (pfad);
		
	// Liet alles Filenamen im angegebenen Directory ein
	String f[] = file.list ();
	
	for (int i = 0; i < f.length; i++)
	    if ((f[i].length() > 14) && (f[i].substring (0, 8).equalsIgnoreCase ("rechnung")) && (f[i].substring (f[i].length() - 4, f[i].length()).equalsIgnoreCase (".txt")))
		{
		StringTokenizer st = new StringTokenizer (f[i], ".");
		if ((st.countTokens() == 3) && (isValidNumber (st.nextToken())) && (isValidKassa (st.nextToken())))
		    ReadRechnungsFile (pfad, f[i]);
		}
	}
	
	
    /**
     * Methode: ReadRechnungsFile.
     * Liet die Rechnungen aus angegebenem File ein.
     * @param pfad - gibt den Pfad an, in dem sich das Rechnungsfile befindet.
     * @param filename - gibt den Filenamen an.
     * @exception MyException Wirft bei Fehler eine Meldung an aufrufende Methode.
     * @see MyException
     */
    void ReadRechnungsFile (String pfad, String filename) throws MyException
	{
	Liste rechnung = new Liste ();
	String s = null;
	int artind = 0;
	ReadFile file;
	boolean err;
	
	// ffnet File
	file = new ReadFile (pfad, filename);

	// Liet bis zum Fileende ein, oder bricht bei mehr als 10 Rechnungsstzen ab
	while ((s = file.readLine()) != null) {
	    // Wenn Leerzeile, nchste Zeile einlesen
	    if (s.length () == 0) continue;
	    
	    err = false;
	    RechnungsElement h = null;
	    // Wandelt Sting in Rechnungsdatensatz um
	    try {h = StringtoRechnung (s);}
	    catch (MyException e) {
		System.out.println (e.getMessage ());
		err = true;}

	    // berprft, ob dieser Artikel in der Rechnung schon vorkommt
	    //  ja: ergnzt den alten Rechnungssatz
	    //  nein: legt neuen Rechnungssatz an
	    if (!err)
		if (RechnungFind (h.artikel, rechnung))
		    ((RechnungsElement) rechnung.getData ()).menge += h.menge;
		else
		    rechnung.add (new RechnungsElement (h.artikel, h.menge));
	    }
	    
	    
	if (!rechnung.isEmpty()) 
	    rechnungsdaten.add (new Rechnung (Integer.parseInt(filename.substring (8, filename.indexOf ("."))), Integer.parseInt (filename.substring (filename.indexOf (".")+1, filename.length () - 4)), rechnung));
	}    


    /**
     * Methode StringtoRechnung.
     * Versucht den bergebenen String in das Datenformat umzuwandeln.
     * @param s - der umzuwandelnde String.
     * @exception MyException Wirft bei Fehler eine Meldung an aufrufende Methode.
     * @see MyException
     * @return Daten - Rechnungsdatensatz.
     */
    private RechnungsElement StringtoRechnung (String s) throws MyException
	{
	int artikel = 0, stck = 0, artnr = 0;
	String help = null;
	
	// Initialisiert Stringteiler und definiert die Trennzeichen
	StringTokenizer st = new StringTokenizer (s, ", ");

	try {
	    if ((st.countTokens () != 2) || (s.charAt (s.length () - 1) == ','))
		throw new MyException ("Falsche Syntax: " + s + " - Datensatz ignoriert.");
	    artikel = Integer.parseInt (st.nextToken());
	    stck = Integer.parseInt (st.nextToken());
	    } 
	catch (NumberFormatException e)
	    {
	    throw new MyException ("Fehler im Datensatz: " + s + " - Datensatz ignoriert.");
	    }
	
	return new RechnungsElement (artikel, stck);
	}		 	 	 
	 
	             
    /**
     * Methode: isValidNumber (String s).
     * berprft, ob der Filename eine gltige Rechnungsnummer enthlt.
     * @param s - Erster Teil des Filenamens.
     * @return true wenn gltig.
     */
    private boolean isValidNumber (String s) {
        try {if (Integer.parseInt (s.substring (8, s.length())) <= 0)
		return false;}
	catch (NumberFormatException e) {return false;}
	return true;
	}
	
	
    /**
     * Methode: isValidKassa (String s).
     * berprft, ob der Filename eine gltige Kassennummer enthlt.
     * @param s - Zweiter Teil des Filenamens.
     * @return true wenn gltig.
     */
    private boolean isValidKassa (String s) {
        try {if (Integer.parseInt (s) <= 0)
		return false;}
	catch (NumberFormatException e) {return false;}
	return true;
	}
	
	
    /**
     * Durchsucht eine Rechnung nach einem bestimmten Artikel.
     * @param art - der gesuchte Artikel.
     * @param d - die zu untersuchende Rechnung.
     * @return boolean - true wenn der Artikel gefunden wurde.
     */
    private boolean RechnungFind (int art, Liste d) {
	if (d.isEmpty ()) return false;
	for (d.gotoStart (); !d.eol (); d.gotoNext())
	    if (((RechnungsElement) d.getData ()).artikel == art)
		return true;
	return false;
	}
    
    
    /**
     * Methode addRechnung ().
     * Fgt neuen Datensatz hinzu.
     * @param nr Die neue Rechnungsnummer.
     * @param kassa Die Kassa, bei der Bezahlt wurde.
     * @param l Die gekauften Artikel.
     */
    public void addRechnung (int nr, int kassa, Liste l) {
	rechnungsdaten.add (new Rechnung (nr, kassa, l));
	Changed ();
	}


    /**
     * Methode addrechnungsElement ().
     * Fgt neuen Datensatz hinzu.
     * @param d die Liste mit den Elementen.
     * @param artikel die neue Artikelnummer.
     * @param menge die konsumierte Menge.
     */
    public void addrechnungsElement (Liste d, int artikel, int menge) {
	d.add (new RechnungsElement (artikel, menge));
	Changed ();
	}


/********************************************************************************************
    Ab hier werden Gruppendaten behandelt
********************************************************************************************/


    /**
     *  Methode: ReadGruppe.
     *  Liet das durch pfad und filename angegeben File ein und Wertet den Inhalt als Gruppen aus.
     *  Falls der Inhalt gltig ist, wird er in einer Liste abgelegt.
     *  @param String pfad: Gibt das Verzeichnis des einzulesenden Files an.
     *  @param String filename: Gibt den Filenamen des einzulesenden Files an.
     *  @see StringtoGruppe
     *  @exception MyException Wirft bei Fehler eine Meldung an aufrufende Methode.
     *	@see MyException
     */
   private void ReadGruppendaten (String pfad, String filename) throws MyException {
	String s = null;
	ReadFile file;

	// Versucht File zu ffnen. Wenn nicht mglich, Abbruch
	file = new ReadFile (pfad, filename);

	// Liet solange Gruppen ein, bis keine mehr vorhanden sind    
	while ((s = file.readLine()) != null) {
	    // Wenn Leerzeile, nchste Zeile einlesen
	    if (s.length () == 0) continue;
	    
	    try {
		// Wandelt einen String ins Gruppenformat um
		StringtoGruppe (s);
		}
	    catch (MyException e) {
		// Fehlermeldung (ErrorDialog)
		System.out.println ("Fehler: " + e.getMessage());
		}
	    }
	    
	file.close ();
	if (gruppendaten.isEmpty())
	    throw new MyException ("Es wurden keine Daten eingelesen - Programmabbruch");
	}
    
    
    /**
     * Methode: String to Gruppe.
     * Versucht den bergebenen String in der Liste "Gruppendaten" als "Gruppe" abzulegen.
     * @param s - der umzuwandelnde String.
     * @exception MyException Wirft bei Fehler eine Meldung an aufrufende Methode.
     * @see MyException
     * @see Gruppe
     */
    private void StringtoGruppe (String s) throws MyException
	{
	int nummer = 0;
	String name = null;
	double preis = 0;
	
	// Initialisiert Stringteiler und definiert die Trennzeichen
	StringTokenizer T = new StringTokenizer (s, ", ");

	try {
	    // berprft, ob die Parameteranzahl korrekt ist
	    if ((T.countTokens () != 2) || (s.charAt (s.length () - 1) == ','))
		throw new MyException ("Falsche Syntax: " + s + " - Datensatz ignoriert.");

	    // Liet die Gruppennummer, wenn nicht einstellig Abbruch
	    name = T.nextToken ();
	    if (name.length() !=1)
		throw new MyException ("Die Gruppennummer " + nummer + " ist ungltig. Sie muss 3stellig sein. Datensatz ignoriert.");
	    nummer = Integer.parseInt (name);
	    
	    if ((nummer < 1) || (nummer > 3))
		throw new MyException ("Die Gruppennummer " + nummer +  " ist ungltig. Sie muss zw. 1 und 3 liegen. Datensatz ignoriert.");

	    // Liet Gruppennamen, berprft, ob der Gruppenname zulssig ist, und bricht bei Fehler ab
	    name = T.nextToken ();
	    if (name.length() > 20)
		throw new MyException ("Gruppenbezeichnung zu lang (max. 20 Zeichen). Datensatz ignoriert.");
	    } 
	catch (NumberFormatException e) {
	    throw new MyException (e.getMessage());
	    }
	    
	// Wenn kein Fehler bei der Umwandlung aufgetreten ist, Datensatz speichern
	gruppendaten.add (new Gruppe (nummer, name));
	}
    }


/**
 * class Artikel.
 * In diesem Format werden die Artikeldaten abgelegt.
 */
class Artikel
    {
    int artikel = 0;
    String name = null;
    double preis = 0;
    
    /**
     * Konstruktor fr ein neues Element.
     * @param nr - die Artikelnummer.
     * @param na - der Artikelname.
     * @param p - der Preis des Artikels.
     */
    Artikel (int nr, String na, double p)
        {
        artikel = nr;
        name = na;
        preis = p;
	}
    }        
	
	
/**
 * Klasse RechnungsDaten.
 * In diesem Format wird eine gesamte Rechnung abgelegt.
 */
class Rechnung
    {
    int rechnungsNr, kassa;
    Liste elements;
	
    /**
     * Konstruktor fr ein neues Element.
     * @param r - die Rechnungsnummer.
     * @param k - die Kassennummer.
     * @param e - die verrechneten Elemente (Artikel).
     */
    Rechnung (int r, int k, Liste e)
        {
        rechnungsNr = r;
        kassa = k;
        elements = e;
        }
    }	
    
    
/**
 * Klasse RechnungsElement.
 * In diesem Format werden die einzelnen Elemente einer Rechnung gespeichert.
 */
class RechnungsElement {
    int artikel, menge;
    
    /**
     * Konstruktor fr ein neues Element.
     * @param a - die Artikelnummer.
     * @param m - die Artikelmenge.
     */
    RechnungsElement (int a, int m)
        {
	artikel = a;
	menge = m;
        }
    }    
    
    
/**
 * class Gruppe.
 * In diesem Format werden die einzelnen Elemente einer Rechnung gespeichert.
 */
class Gruppe {
    int artikel;
    String name;
    
    /**
     * Konstruktor fr ein neues Element.
     * @param a - die Artikelgruppennummer (1-3).
     * @param n - der Artikelgruppenname.
     */
    Gruppe (int a, String n)
        {
        artikel = a;
        name = n;
	}
    }    
