import java.awt.*;


/**
 * class Diagramm.
 * Oberklasse fr Kreis- und Balkendiagramm.
 * @see Kreisdiagramm
 * @see Balkendiagramm
 */
abstract class Diagramm {
    protected int hspace = 0, xbez = 20, breite = 20, maxX, maxY, ybez;
    protected MyColors m = new MyColors ();
    protected Liste werte;
    protected Liste texte;
    protected FontMetrics fm;
    protected Graphics g;
    protected Dimension groesse;
    protected Canvas output;
    protected ScrollPane scrollpane;

    
    /**
     * Konstruktor.
     * bernimmt die bergebenen Referenzen und startet die Zeichnung des Diagrammes.
     * @param gr die Graphicsreferenz.
     * @param data die Diagarammdaten (Beschriftung und Werte).
     * @see Diagrammdaten
     * @param sp der ScrollPane, in dem der Canvas liegt.
     * @param op der Canvas, in dem die Ausgabe erfolgt.
     */
    public Diagramm (Graphics gr, Diagrammdaten data, ScrollPane sp, Canvas op) {
	// Scrollpane, Graphics, Output (Canvas) und Daten bernehmen
	scrollpane = sp;
	g = gr;
	output = op;
	texte = data.getNames ();
	werte = data.getValues ();
	
	// Diagramm ausgeben
	paintDiagramm ();
	}
	
	
    /**
     * methode fitFont ().
     * Berechnet die optimale Schriftgrsse.
     * @param max gibt die max. Grsse der Schrift in Pixel an.
     * @return Font die optimale Schriftart.
     */
    protected Font fitFont (int max) {
	FontMetrics fm;
	Font font = null;
	
	int groesse = 20;
	while (groesse > 6) {
	    font = new Font ("Serif", Font.PLAIN, groesse);
	    fm = g.getFontMetrics (font);
	    if (fm.getHeight () + 5 < max) break;
	    groesse--;
	    }
	return font;
	}
	
	
    /**
     * abstract methode paintDiagramm ().
     * Zeichnet das jeweilige Diagramm.
     */
    public abstract void paintDiagramm ();
    }
    

/**
 * class Balkendiagramm extends Diagramm.
 * Hier werden Balkendiagramme ausgegeben.
 * @see Diagramm
 */	
class Balkendiagramm extends Diagramm {
    
    /**
     * Konstruktor.
     * Startet Superkonstruktor.
     * @param g die Graphicsreferenz.
     * @param data die Diagarammdaten (Beschriftung und Werte).
     * @see Diagrammdaten
     * @param scrollpane der ScrollPane, in dem der Canvas liegt.
     * @param ootput der Canvas, in dem die Ausgabe erfolgt.
     */
    public Balkendiagramm (Graphics g, Diagrammdaten data, ScrollPane scrollpane, Canvas output) {
        super (g, data, scrollpane, output);
        }
    
    /**
     * methode paintDiagramm ().
     * Zeichnet ein Balkendiagramm.
     */
    public void paintDiagramm () {
        int balken = 0;
        double fact = 0.0;

	// Berechnet Werte Grsse und Einteilung des Canvas
        groesse = scrollpane.getViewportSize ();
        maxX = groesse.width * 7 / 10;
        maxY = groesse.height * 7 / 10;
        ybez = groesse.height / 10;	    
	
	// Lscht altes Diagramm
        g.clearRect (0, 0, groesse.width, groesse.height);
	
	// Setzt Schriftgrsse ein und Berechnet die Fontmetrics
        g.setFont (fitFont (ybez));
        fm = g.getFontMetrics ();
	    	    
	// Berechnet die max. Breite einer Bezeichnung + hchsten Balken
        for (texte.gotoStart (), werte.gotoStart (); !texte.eol (); texte.gotoNext (), werte.gotoNext ())
	    if (Integer.parseInt ((String) werte.getData ()) != 0) {
		balken ++;
		if (fm.stringWidth ((String) texte.getData ()) + 10 > hspace)
		    hspace = fm.stringWidth((String) texte.getData ()) + 10;
		if (fm.stringWidth((String) werte.getData ()) + 10 > hspace)
		    hspace = fm.stringWidth ((String) werte.getData ()) + 10;
		if (fact < (1.0 * Integer.parseInt ((String) werte.getData ())/(maxY)))
		    fact = 1.0 * Integer.parseInt ((String) werte.getData ())/(maxY);
		}
	    
	if (fact == 0)
	    fact = 1;
	if (hspace <= breite)
	    hspace = breite * 2;
	    
	if ((balken * hspace + 2 * xbez) < (groesse.width + 20)) 
	    // Passt den Canvas an das auszugebende Diagramm an
	    output.setSize (groesse.width, groesse.height);
	else 
	    // Passt den Canvas an das auszugebende Diagramm an
	    output.setSize (balken * hspace + 2 * xbez, groesse.height);
	    
	scrollpane.doLayout ();
	    
	// Gibt das Diagramm aus    	    
	int i = 0;
	for (texte.gotoStart (), werte.gotoStart (); !texte.eol (); texte.gotoNext (), werte.gotoNext ())
	    if (Integer.parseInt ((String) werte.getData ()) != 0) {
		g.setColor (m.getNext ());
		g.fillRect(xbez + hspace*i + hspace/2 - breite/2, maxY + ybez*3/2 - (int)Math.round(Integer.parseInt ((String) werte.getData ()) / fact), breite, (int)Math.round(Integer.parseInt ((String) werte.getData ()) / fact));
		g.setColor (Color.black);
		drawStringZentriert ((String) texte.getData (), xbez + hspace*i, maxY + ybez*5/2, hspace);
		drawStringZentriert ((String) werte.getData (), xbez + hspace*i, maxY + ybez - (int)Math.round(Integer.parseInt ((String) werte.getData ()) / fact), hspace);
		g.drawRect(xbez + hspace*i + hspace/2 - breite/2, maxY + ybez*3/2 - (int)Math.round(Integer.parseInt ((String) werte.getData ()) / fact), breite, (int)Math.round(Integer.parseInt ((String) werte.getData ()) / fact));
		i++;
		}
	}
	
	
    /**
     * methode drawStringZentriert ().
     * Gibt einen String zentriert aus.
     * @param s der auszugebende String.
     * @param x die x-Position des Strings.
     * @param y die y-Position des Strings.
     * @param m die Grsse des grssten auszugebenden Strings
     */
    private void drawStringZentriert (String s, int x, int y, int m) {
	FontMetrics fm = g.getFontMetrics ();
	int w = fm.stringWidth (s);
	
	g.drawString (s, x + (int)m/2 - (int)w/2, y);
	}
    }

        
/**
 * class Kreisdiagramm extends Diagramm.
 * Hier werden Kreisdiagramme ausgegeben.
 * @see Diagramm
 */	
class Kreisdiagramm extends Diagramm {
    
    /**
     * Konstruktor.
     * Startet Superkonstruktor.
     * @param g die Graphicsreferenz.
     * @param data die Diagarammdaten (Beschriftung und Werte).
     * @see Diagrammdaten
     * @param scrollpane der ScrollPane, in dem der Canvas liegt.
     * @param ootput der Canvas, in dem die Ausgabe erfolgt.
     */
    public Kreisdiagramm (Graphics g, Diagrammdaten data, ScrollPane scrollpane, Canvas output) {
	super (g, data, scrollpane, output);
	}
    
    /**
     * methode paintDiagramm ().
     * Zeichnet ein Kreisdiagramm.
     */
    public void paintDiagramm () {
        int ges = 0, radius; 
	double start = 0.0;

	// Berechnet Werte Grsse und Einteilung des Canvas
        groesse = scrollpane.getViewportSize ();
	maxX = groesse.width * 7 / 10;
        maxY = groesse.height * 7 / 10;
        ybez = groesse.height / 10;	    

	// Lscht altes Diagramm
        g.clearRect (0, 0, groesse.width, groesse.height);

	// Setzt Schriftgrsse ein und Berechnet die Fontmetrics
        g.setFont (fitFont (ybez));
        fm = g.getFontMetrics ();
	    	    
	// Berechnet Werte Grsse und Einteilung des Canvas
        radius = Math.min (maxX, maxY)/2;
	    
	// Berechnet die Gesamtanzahl der Artikel
        for (werte.gotoStart (); !werte.eol (); werte.gotoNext ())
	    if (Integer.parseInt ((String) werte.getData ()) != 0)
		ges += Integer.parseInt ((String) werte.getData ());
	    
	if (hspace <= breite)
	    hspace = breite * 2;
		
	// Passt den Canvas an das auszugebende Diagramm an
	output.setSize (groesse.width, groesse.height);
	scrollpane.doLayout ();
	    
	// Gibt das Diagramm aus    	    
	g.setColor (Color.black);		
	g.fillOval (groesse.width/2 - radius - 2, groesse.height/2 - radius - 2, radius*2 + 4, radius*2 + 4);
	
        for (werte.gotoStart (), texte.gotoStart (); !werte.eol (); werte.gotoNext (), texte.gotoNext ())
	    if (Integer.parseInt ((String) werte.getData ()) != 0) {
		double bogen = 360.0 * Integer.parseInt ((String) werte.getData ()) / ges;
		g.setColor (m.getNext ());
		g.fillArc (groesse.width/2 - radius, groesse.height/2 - radius, radius*2, radius*2, (int) Math.round (start), (int) Math.round (bogen));
		g.setColor (Color.black);		
		drawKreisBeschriftung ((String) texte.getData (), groesse.width/2 - radius, groesse.height/2 - radius, (int) Math.round (1.5 * radius), (int) Math.round (start), (int) Math.round (bogen));
		start += bogen;
	        }
	}
	
	
    /**
     * Methode drawKreisBeschriftung.
     * Berechnet die Koordinate fr die Beschriftung der Kreissegmente und
     * ruft die Ausgabe des Textes auf. Ausserdem werden die Randlinien der Segmente gezeichnet.
     * @param s der auszugebende String.
     * @param x die x-Koordinate des Kreises (links).
     * @param y die y-Koordinate des Kreises (oberen).
     * @param r der Radius des Kreises.
     * @param wink der Bogenwinkel des Kreises.
     * @param endwink der Endwinkel des Kreises.
     */
    private void drawKreisBeschriftung (String s, int x, int y, int r, int wink, int endwink) {
        int centerX = x + r;
        int centerY = y + r;
        int xbe, ybe = xbe = 0;
        double store1 = 1.0*wink;
        double store2 = 1.0*endwink;
        double store3 = 2.0*r;
        double store4 = 2.0*r;
	    
        double BWink = 1.0*(((store1+(store2/2))/360.0)*(Math.PI)*2.0);
        double LWink = 1.0*((store1/360.0)*(Math.PI)*2.0);
        double sinLWink = Math.sin (LWink);
        double cosLWink = Math.cos (LWink);
        double sinBWink = Math.sin (BWink);
        double cosBWink = Math.cos (BWink);
        
        // Sektor von 0 bis 90 Grad 	    
        if (BWink <= Math.PI/2) {		  
	    xbe = (int) (centerX + cosBWink*store3/2);
	    ybe = (int) (centerY - sinBWink*store4/2);
	    drawText (s, xbe, ybe, false, true);
	    g.drawLine (centerX, centerY, (int) (centerX + cosLWink*store3/2), (int) (centerY - sinLWink*store4/2));
	    }
	    
        // Sektor von 90 bis 180 Grad 	    
	else if (BWink <= Math.PI && BWink > Math.PI/2) {	   
	    xbe = (int) (centerX + cosBWink*store3/2);
	    ybe = (int) (centerY - sinBWink*store4/2);
	    drawText (s, xbe, ybe, true, true);
	    g.drawLine (centerX, centerY, (int) (centerX + cosLWink*store3/2), (int) (centerY - sinLWink*store4/2));
	    }
		
	// Sektor von 180 bis 270 Grad 	    
	else if (BWink <= 3*Math.PI/2 && BWink > Math.PI) {	    
	    xbe = (int) (centerX + cosBWink*store3/2);
	    ybe = (int) (centerY - sinBWink*store4/2);
	    g.drawLine (centerX, centerY, (int) (centerX + cosLWink*store3/2), (int) (centerY - sinLWink*store4/2));
	    drawText (s, xbe, ybe, true, false);
	    }
		
	// Sektor von 270 bis 360 Grad 	    
	else if (BWink > 3*Math.PI/2 && BWink <= 2*Math.PI ) {			    
	    xbe = (int) (centerX + cosBWink*store3/2);
	    ybe = (int) (centerY - sinBWink*store4/2);
	    g.drawLine (centerX, centerY, (int) (centerX + cosLWink*store3/2), (int) (centerY - sinLWink*store4/2));
	    drawText (s, xbe, ybe, false, false);
	    }
	}	    
    
    
    /**
     * Methode drawText ().
     * Gibt bergebenen String mit der gewnschten Ausrichtung aus.
     *	- Links-/Rechtsbndig.
     *  - An der oberen/unteren Kante ausgerichtet.
     * @param s der auszugebende String.
     * @param x die x-Position des Strings.
     * @param y die y-Position des Strings.
     * @param left true, wenn der String linksbndig ausgegeben werden soll.
     * @param top true, wenn der String an der oberen Kante ausgerichtet sein soll.
     */
    private void drawText (String s, int x, int y, boolean left, boolean top) {
	FontMetrics fm = g.getFontMetrics ();
	int w = fm.stringWidth (s);
	int h = fm.getMaxAdvance();
	int c = fm.charWidth (' ');
	
	if (left)
	    if (top)
		g.drawString (s, x - w - c, y);
	    else
		g.drawString (s, x - w - c, y + h);
	else
	    if (top)
		g.drawString (s, x + c, y);
	    else
		g.drawString (s, x + c, y + h);
	}
    }