/**
 * Abstract Class Diagrammdaten.
 * Sie garantiert die Daten, die zur Anzeige des Diagramms bentigt werden.
 * Aus ihr werden 3 Klassen abgeleitet: KassaDaten, GruppenDaten, ArtikelDaten.
 * Diese berechnen jeweils die bentigten Daten (Beschriftung, Anzahl).
 * @version        1.0 15 Jan 1999
 * @author         Sascha Nemecek
 */
abstract class Diagrammdaten {
    Daten daten = null;


    /**
     * Konstruktor.
     * @param daten Die gesamten Daten. 
     */
    public Diagrammdaten (Daten daten) {
	this.daten = daten;
	}

        
    /**
     * Abstract Method getNames.
     * Liefert die Beschriftung (Artikel-, Gruppenname oder Kassa) fr Diagramm.
     * @return Liste mit den Beschriftungen
     */
    abstract public Liste getNames ();


    /**
     * Abstract Method getValues.
     * Liefert die Anzahl (Artikel-, Gruppenmenge oder Stk/Kassa) fr Diagramm.
     * @return Liste mit den Werten
     */
    abstract public Liste getValues ();
    
    
    /**
     * Abstract Method CountOccurence.
     * Berechnet die Gesamtmenge einer Artikelgruppe/eines Artikels in einer Rechnung.
     * @param art die/der gesuchte Artikelgruppe/Artikel.
     * @param d die zu untersuchende Rechnung.
     * @return die Menge der gesuchten Gruppe/Artikel in dieser Rechnung.
     */
    abstract protected int CountOccurence (int art, Liste d);
    
    
    /**
     * getArtikelMenge.
     * Berechnet die Gesamtmenge eines Artikels/einer Artikelgruppe in allen Rechnungen.
     * @param art der gesuchte Artikel.
     * @param d Liste mit den Rechnungen.
     * @return die Menge des/der gesuchten Artikels/Artikelgruppe in allen Rechnungen.
     */
    protected int getArtikelMenge (int art, Liste d) {
	int occ = 0;
	for (d.gotoStart (); !d.eol (); d.gotoNext ())
	    occ += CountOccurence (art, ((Rechnung) d.getData ()).elements);
	return occ;
	}		
    }
    


/**
 * Class KassaDaten.
 * Liefert die bentigten Daten zur Anzeige der verkauften Artikel pro Kassa.
 * @version        1.0 15 Jan 1999.
 * @author         Sascha Nemecek.
 */
class KassaDaten extends Diagrammdaten {
    public KassaDaten (Daten daten) {
	super (daten);
	}
	
	
    /**
     * Method getNames.
     * Liefert die Beschriftung (Kassennummer) fr Diagramm.
     * @return Liste mit den Kassennummern.
     */
    public Liste getNames () {
	Liste grp = new Liste ();
	Liste d = daten.rechnungsdaten;
	
	for (d.gotoStart (); !d.eol (); d.gotoNext ())
	
	    // berprft, ob diese Kassa in der Liste schon vorkommt
	    //  ja: lsst die List
	    //  nein: legt neues Element an
	    if (!KassaExists (grp, ((Rechnung) d.getData ()).kassa))
		grp.add ("" + ((Rechnung) d.getData ()).kassa);
	return grp;
	}
    
    
    private boolean KassaExists (Liste grp, int kassa) {
	for (grp.gotoStart (); !grp.eol (); grp.gotoNext ())
	    if (((String) grp.getData ()).equals ("" + kassa))
		return true;
	return false;
	}
	
	
    /**
     * Method getValues.
     * Liefert die Anzahl der verkauften Artikel pro Kassa fr Diagramm.
     * @return Liste mit den verkauften Stck pro Kassa.
     */
    public Liste getValues () {
	Liste names = getNames ();
	Liste grp = new Liste ();
	
	for (names.gotoStart (); !names.eol (); names.gotoNext ())
	    grp.add ("" + getArtikelMenge (Integer.parseInt ((String) names.getData ()), daten.rechnungsdaten));
	return grp;
	}
	
    
    /**
     * getArtikelMenge.
     * Overwrites Diagrammdaten.getArtikelMenge.
     * Berechnet die Gesamtmenge eines Artikels in einer Kassa.
     * @param kas die gesuchte Kassa.
     * @param d Liste mit den Rechnungen.
     * @return die Menge der gekaufen Artikel pro Kassa.
     */
    protected int getArtikelMenge (int kas, Liste d) {
	int occ = 0;
	for (d.gotoStart (); !d.eol (); d.gotoNext ())
	    if (((Rechnung) d.getData ()).kassa == kas)
		occ += CountOccurence (0, ((Rechnung) d.getData ()).elements);
	return occ;
	}		
	
	
    /**
     * CountOccurence.
     * Berechnet die Gesamtmenge der verkauften Artikel in einer Rechnung.
     * @param art ohne Funktion.
     * @param d die zu untersuchende Rechnung.
     * @return die Menge der Artikel in dieser Rechnung.
     */
    protected int CountOccurence (int art, Liste d) {
	int i = 0;
	for (d.gotoStart (); !d.eol (); d.gotoNext ())
	    i += ((RechnungsElement)d.getData ()).menge;
	return i;
	}
    }


/**
 * Class GruppenDaten.
 * Liefert die bentigten Daten zur Anzeige der verkauften Artikel pro Kassa.
 * @version        1.0 15 Jan 1999.
 * @author         Sascha Nemecek.
 */
class GruppenDaten extends Diagrammdaten {

    public GruppenDaten (Daten daten) {
	super (daten);
	}
	
	
    /**
     * Method getNames.
     * Liefert die Beschriftung (Gruppennamen) fr das Diagramm.
     * @return Liste mit den Gruppennamen.
     */
    public Liste getNames () {
	Liste grp = new Liste ();
	Liste d = daten.gruppendaten;
	
	for (d.gotoStart (); !d.eol (); d.gotoNext ())
	    grp.add (((Gruppe) d.getData ()).name);
	return grp;
	}
	
	
    /**
     * Method getValues.
     * Liefert die Anzahl der verkauften Artikel pro Artikelgruppe fr das Diagramm.
     * @return Liste mit den verkauften Artikel pro Artikelgruppe.
     */
    public Liste getValues () {
	Liste grp = new Liste ();
	Liste g = daten.gruppendaten;
	
	for (g.gotoStart (); !g.eol (); g.gotoNext ())
	    grp.add ("" + getArtikelMenge (((Gruppe) g.getData ()).artikel, daten.rechnungsdaten));
	return grp;
	}


    /**
     * CountOccurence.
     * Berechnet die Gesamtmenge einer Artikelgruppe in einer Rechnung.
     * @param art die gesuchte Artikelgruppe.
     * @param d die zu untersuchende Rechnung.
     * @return die Menge der gesuchten Gruppe in dieser Rechnung.
     */
    protected int CountOccurence (int art, Liste d) {
	int i = 0;
	for (d.gotoStart (); !d.eol (); d.gotoNext ())
	    if (art == ((RechnungsElement)d.getData ()).artikel/100)
		i += ((RechnungsElement)d.getData ()).menge;
	return i;
	}
    }


/**
 * Class ArtikelDaten.
 * Liefert die bentigten Daten zur Anzeige der verkauften Artikel.
 * @version        1.0 15 Jan 1999.
 * @author         Sascha Nemecek.
 */
class ArtikelDaten extends Diagrammdaten {
    public ArtikelDaten (Daten daten) {
	super (daten);
	}
	
	
    /**
     * Method getNames.
     * Liefert die Beschriftung (Artikelnamen) fr das Diagramm.
     * @return Liste mit den Artikelnamen.
     */
    public Liste getNames () {
	Liste grp = new Liste ();
	Liste d = daten.artikeldaten;
	
	for (d.gotoStart (); !d.eol (); d.gotoNext ())
	    grp.add (((Artikel) d.getData ()).name);
	return grp;
	}
	
	
    /**
     * Method getValues.
     * Liefert die Anzahl der verkauften Artikel fr das Diagramm.
     * @return Liste mit der Anzahl der verkauften Artikel.
     */
    public Liste getValues () {
	Liste grp = new Liste ();
	Liste g = daten.artikeldaten;
	
	for (g.gotoStart (); !g.eol (); g.gotoNext ())
	    grp.add ("" + getArtikelMenge (((Artikel) g.getData ()).artikel, daten.rechnungsdaten));
	return grp;
	}


    /**
     * CountOccurence.
     * Berechnet die Gesamtmenge eines Artikels in einer Rechnung.
     * @param art der gesuchte Artikel.
     * @param d die zu untersuchende Rechnung.
     * @return die Menge des gesuchten Artikels in dieser Rechnung.
     */
    protected int CountOccurence (int art, Liste d) {
	int i = 0;
	for (d.gotoStart (); !d.eol (); d.gotoNext ())
	    if (art == ((RechnungsElement)d.getData ()).artikel)
		i += ((RechnungsElement)d.getData ()).menge;
	return i;
	}
    }
