import java.awt.*;
import java.awt.event.*;
import java.util.*;


/**
 * Dialog MyDialog.
 * Abstrakte Klasse fr Dialoge
 */
abstract class MyDialog extends Dialog {
    
    /**
     * Konstruktor MyDialog ().
     * ffnet einen neuen Dialog.
     * @param p der Mutterframe.
     * @param title der Dialogname.
     */
    MyDialog (Frame p, String title) {
	super (p, title);
	}
	
    /**
     * Konstruktor MyDialog ().
     * ffnet einen neuen Dialog.
     * @param p der Mutterframe.
     * @param title der Dialogname.
     * @param mod true, wenn der Dialog Modal ist.
     */
    MyDialog (Frame p, String title, boolean mod) {
	super (p, title, mod);
	}
    
    
    /**
     * abstrakte Methode cancel ().
     * Beendet den Dialog und kehrt zurck zum Hauptframe.
     */
    abstract public void cancel ();
    }


/**
 * class Dialogschliesser.
 * Schliesst den Dialog.
 */
class Dialogschliesser extends WindowAdapter {

    /**
     * Referenz auf den Dialog, dem der Dialogschliesser gehrt.
     */
    MyDialog dialog;

    /**
     * Konstruktor.
     */
    Dialogschliesser (MyDialog d) {
	super ();
	this.dialog = d;
	}

    /**
     * Methode windowClosing ().
     * startet die Schliessung des Fensters.
     */
    public void windowClosing (WindowEvent e) {
	dialog.cancel ();}
    }


/**
 * Dialog Bearbeiten.
 * Ermglicht das Bearbeiten einer oder mehrerer Rechnungen.
 * Zur Zeit ist nur das Lschen von Artikeln in Rechnungen und das bearbeiten von Artikeln mglich.
 */
class DialogBearbeiten extends MyDialog implements ActionListener, ItemListener, Observer {
    private Choice choiceRechnungen = new Choice();
    private Panel pEingabe = new Panel ();
    private List lElements = new List (5, false);
    private Button bNeu = new Button ("Neu");
    private Button bEd = new Button ("Bearbeiten");
    private Button bDel = new Button ("Lschen");
    private Button bOk = new Button ("OK");
    private TextField tfArt = new TextField ();
    private Daten daten;
    private Liste rechnungsDaten;
    private Frame parent;


    /**
     * Konstrukor.
     * Hier wird der Dialog initialisiert.
     */
    DialogBearbeiten (Frame p, String titel, boolean modal, Daten d) {
	super(p,titel);
	parent = p;
	daten = d;
	rechnungsDaten = daten.rechnungsdaten;
	
	setBackground(Color.lightGray);
	setResizable(false);
	setLayout(new BorderLayout());
	
	// Liste fllen
        for (rechnungsDaten.gotoStart (); !rechnungsDaten.eol (); rechnungsDaten.gotoNext ())
	    choiceRechnungen.add ("" + ((Rechnung) rechnungsDaten.getData()).rechnungsNr);
	
	// Fllt die Liste    
	fillList ();
		    	    
	pEingabe.setLayout(new GridLayout(3,1));
	pEingabe.add(bNeu);
	pEingabe.add(bEd);
	pEingabe.add(bDel);
	
	choiceRechnungen.addItemListener(this);
	bOk.addActionListener(this);
	bNeu.addActionListener(this);
	bDel.addActionListener(this);
	bEd.addActionListener(this);
	
	lElements.addActionListener (this);
	lElements.addItemListener (this);
	
	add ("North", choiceRechnungen);
	add ("Center", lElements);
	add ("East", pEingabe);
	add ("South", bOk);

	// WindowListener zum Schlieen des Dialoges
	addWindowListener(new Dialogschliesser (this));
	pack();
	}
	
    
    /**
     * Methode: Update.
     * Fhrt bei einer Vernderung des zu berwachenden Objektes die fillListmethode des Dialoges aus.
     * @param o - das berwachte Objekt.
     * @param arg - die Vernderung.
     */
    public void update(Observable o, Object arg) {
	fillList ();
	}
    
	    
    /**
     * methode fillList.
     * Fllt die Liste mit dem Inhalt einer Rechnung.
     */    
    private void fillList () {
	String sRechnung;
	Liste rechnungsElemente = null;
	    
	// selektierte Rechnungsnummer holen
	if ((sRechnung = choiceRechnungen.getSelectedItem()) == null)
	    cancel ();
	
	lElements.removeAll ();
	
	// Sucht Rechnung
	for (rechnungsDaten.gotoStart (); !rechnungsDaten.eol (); rechnungsDaten.gotoNext ())
	    if (((Rechnung) rechnungsDaten.getData()).rechnungsNr == Integer.parseInt (sRechnung)) {
		rechnungsElemente = ((Rechnung) rechnungsDaten.getData()).elements;
	
		for (rechnungsElemente.gotoStart (); !rechnungsElemente.eol (); rechnungsElemente.gotoNext ())
		    lElements.add (daten.getArtikelName (((RechnungsElement) rechnungsElemente.getData ()).artikel) + "    " + ((RechnungsElement) rechnungsElemente.getData ()).menge);
		}
	}


    /**
     * Methode cancel ().
     * Beendet den Dialog und kehrt zurck zum Hauptframe.
     */
    public void cancel() {
	setVisible(false);
	dispose();
	((Window)getParent()).toFront();
	getParent().requestFocus();
	}


    /**
     * Methode actionPerformed ().
     * Wertet die ActionEvent aus und reagiert je nach Event.
     * @param e das ausgelte ActionEvent.
     */
    public void actionPerformed (ActionEvent e) {		    
	// Beendet Bearbeitung
	if (e.getActionCommand().equals("OK")) cancel();
	
	// noch nicht ausprogrammiert
	else if (e.getActionCommand().equals("Neu")) {
	    String r;
	    DialogNeuesElement dNeu;
	
	    // selektierte Rechnungsnummer holen
	    if ((r = choiceRechnungen.getSelectedItem()) == null)
		cancel ();
	    else {
		dNeu = new DialogNeuesElement (parent, r, daten);
		dNeu.setVisible (true);
		}
	    }
	
	// Startet einen neuen Dialog zum verndern des ausgewhlten Artikels
	else if (e.getActionCommand().equals("Bearbeiten")) {
	    String r, s = lElements.getSelectedItem ();
	    DialogEditElement dEd;
	
	    // selektierte Rechnungsnummer holen
	    if ((r = choiceRechnungen.getSelectedItem()) == null)
		cancel ();
	
	    if (s != null) {
		dEd = new DialogEditElement (parent, s, r, daten);
		dEd.setVisible (true);
		}
	    }
	
	// Lscht den ausgewhlten Artikeleintrag einer Rechnung
	else if (e.getActionCommand().equals("Lschen")) lschen();
	}
	
    
    /**
     * methode itemStateChanged.
     * Verarbeitet die Item Events des Choice- und Listobjekts..
     */
    public void itemStateChanged (ItemEvent e) {
	// Wenn eine andere Rechnung ausgewhlt wurde - aktualisieren
	if (e.getSource () != lElements)
	    fillList ();
	}

		
    /**
     * Lscht das selektierte Listenelement.
     */
    public void lschen() {
	String sElement, sRechnung;
	Liste rechnungsElemente = null;
	
	// selektierte Rechnungsnummer holen
	if ((sRechnung = choiceRechnungen.getSelectedItem()) == null)
	    cancel ();
	
	// Sucht Rechnung
	for (rechnungsDaten.gotoStart (); !rechnungsDaten.eol (); rechnungsDaten.gotoNext ())
	    if (((Rechnung) rechnungsDaten.getData()).rechnungsNr == Integer.parseInt (sRechnung))
		rechnungsElemente = ((Rechnung) rechnungsDaten.getData()).elements;
		
	// selektierten Artikel holen
	if ((sElement = lElements.getSelectedItem()) != null)
	    {	    
	    // Rechnungsnummer aus Choice entfernen
	    lElements.remove (sElement);
	    
	    // Rechnungsnummer aus Daten entfernen & Observer benachrichtigen
	    for (rechnungsElemente.gotoStart (); !rechnungsElemente.eol (); rechnungsElemente.gotoNext ())
		if (sElement.equals (daten.getArtikelName (((RechnungsElement) rechnungsElemente.getData ()).artikel) + "    " + ((RechnungsElement) rechnungsElemente.getData ()).menge)) {
		    rechnungsElemente.remove ();
		    daten.Changed ();
		    }
    	    }
    	}
    }    
    
    
    
/**
 * Klasse DialogEditElement.
 * Ermglicht das Editieren eines Rechnungselements.
 * Dies geschieht ber einen modalen Dialog.
 */
class DialogEditElement extends MyDialog implements ActionListener {
    Button bOk = new Button ("OK");
    Button bCancel = new Button ("Cancel");
    Panel p = new Panel ();
    TextField tf = new TextField ("0", 1);
    Label lb = new Label ();
    Liste artikelDaten;
    Liste rechnungsDaten;
    Daten daten;
    String art;
    String nr;
    

    /**
     * Kostruktor.
     * ffnet den modalen Dialog.
     * @param parent Referenz auf das Mutterfenster.
     * @param s der zu verndernde Artikel.
     * @param n die Rechnung die den Artikel beinhaltet.
     * @param d die Daten.
     */
    DialogEditElement (Frame parent, String s, String n, Daten d) {
	super (parent, "Edit", true);
	artikelDaten = d.artikeldaten;
	rechnungsDaten = d.rechnungsdaten;
	daten = d;
	art = s;
	nr = n;

	// Setzt Parameter fr Dialog	
        setBackground(Color.lightGray);
        setResizable(false);
        setLayout(new BorderLayout());
	
	// Initialisiert die ActionListener
	bOk.addActionListener (this);
	bCancel.addActionListener (this);
	lb.setText ("Stck: " + s.substring (0, s.indexOf (" ")));
	tf.setText (s.substring (s.indexOf (" ") + 4, s.length ()));
	
	// Fgt buttons in Panel ein
	p.setLayout (new GridLayout(1, 2));
	  p.add (bOk);
	  p.add (bCancel);
	
	// Positioniert die Objekte im Dialog
	add ("Center", tf);
	add ("North", lb);
	add ("South", p);	

        // WindowListener zum Schlieen des Dialoges
        addWindowListener(new Dialogschliesser (this));
	
	// Richtet Objekte ein
        pack();
        }


    /**
     * Methode cancel ().
     * Beendet den Dialog und kehrt zurck zum Mutterfenster..
     */
    public void cancel() {
	setVisible(false);
	dispose();
	((Window)getParent()).toFront();
	getParent().requestFocus();
	}		        	    	    


    /**
     * Verarbeitet ActionEvents.
     * @param e das ausgelte Event.
     */
    public void actionPerformed (ActionEvent e) {		    
	// Beendet Bearbeitung, wenn Eingabe korrekt
	if (e.getActionCommand().equals ("OK")) 
	    {
	    boolean err = false;
	    int menge = 0;
	    try {menge = Integer.parseInt (tf.getText ());}
	    catch (NumberFormatException ex) {err = true;}
	    
	    // Wenn kein Fehler aufgetreten ist - sichern
	    if ((menge > 0) && !err) {
		Liste rechnungsElemente = null;
		
		// Sucht Rechnung
		for (rechnungsDaten.gotoStart (); !rechnungsDaten.eol (); rechnungsDaten.gotoNext ())
		    if (((Rechnung) rechnungsDaten.getData()).rechnungsNr == Integer.parseInt (nr))
			rechnungsElemente = ((Rechnung) rechnungsDaten.getData()).elements;
		
		// Sucht Artikel in Rechnung und verndert seine Menge
		for (rechnungsElemente.gotoStart (); !rechnungsElemente.eol (); rechnungsElemente.gotoNext ())
		    if (art.equals (daten.getArtikelName (((RechnungsElement) rechnungsElemente.getData ()).artikel) + "    " + ((RechnungsElement) rechnungsElemente.getData ()).menge)) {
			((RechnungsElement) rechnungsElemente.getData ()).menge = menge;
			daten.Changed ();
			}
			
		cancel ();	  
		}
	    }
	    
	// Beendet Bearbeitung sofort - ohne nderung
	if (e.getActionCommand().equals ("Cancel")) 
	    cancel ();
	}        
    }
    
    
/**
 * Dialog Lschen.
 * Lschen einer oder mehrerer Rechnungen.
 */
class DialogLoeschen extends MyDialog implements ActionListener, ItemListener {
    List choiceRechnungen = new List (5, false);
    Panel pEingabe = new Panel();
    Button bOk = new Button("Lschen");
    Button bCancel = new Button ("Cancel");
    Daten d;
    Liste daten;

    
    /**
     * Konstrukor.
     * Hier wird der Dialog initialisiert.
     */
    DialogLoeschen (Frame parent, String titel, boolean modal, Daten v) {
	super(parent,titel);
	d = v;
	daten = d.rechnungsdaten;
	setBackground(Color.lightGray);
	setResizable(false);
	
	// Liste fllen
        for (daten.gotoStart (); !daten.eol (); daten.gotoNext ())
	    choiceRechnungen.add ("" + ((Rechnung) daten.getData()).rechnungsNr);
	    
	pEingabe.setLayout(new GridLayout(2,1));
	  pEingabe.add(bOk);
	  pEingabe.add(bCancel);
	  bOk.addActionListener(this);
	  bCancel.addActionListener(this);
	
	choiceRechnungen.addItemListener(this);
	add("Center", choiceRechnungen);
	add("South", pEingabe);

	// WindowListener zum Schlieen des Dialoges
	addWindowListener(new Dialogschliesser (this));
	pack();
	}


    /**
     * Beendet den Dialog und kehrt zurck zum Hauptframe.
     */
    public void cancel() {
	setVisible(false);
	dispose();
	((Window)getParent()).toFront();
	getParent().requestFocus();
	}
	
	        
    /**
     * Wertet die ActionEvent aus und reagiert je nach Event.
     */	    
    public void actionPerformed (ActionEvent e) {
	// lschen aufrufen
	if (e.getActionCommand().equals("Lschen")) lschen();
	
	// Zurck zum Hauptframe
	if (e.getActionCommand().equals("Cancel")) cancel();    
	}
	
	
    /**
     * methode itemStateChanged.
     * Verarbeitet die Item Events der Listbox.
     */
    public void itemStateChanged (ItemEvent e) {}

	    	    
    /**
     * Methode lschen.
     * Lscht die ausgewhlte Rechnung.
     */
    void lschen() {
	// selektierte Rechnungsnummer holen
	String sRechnung = choiceRechnungen.getSelectedItem();
	    
	if (sRechnung != null)
	    {	    
	    // Rechnungsnummer aus Choice entfernen
	    choiceRechnungen.remove (sRechnung);			
	    
	    // Rechnungsnummer aus Daten entfernen
	    for (daten.gotoStart (); !daten.eol (); daten.gotoNext ())
		if (((Rechnung) daten.getData()).rechnungsNr == Integer.parseInt (sRechnung))
		    {
		    daten.remove ();
		    d.Changed ();
		    }
		
	
	    if (daten.isEmpty ())
		cancel ();
	    }
    	}
    }    
    
    
/**
 * Dialog Neu.
 * Fgt eine Rechnung hinzu.
 * Noch nicht funktionstchtig, da noch die Ausprogrammierung der Artikelauswahl fehlt.
 */
class DialogNeu extends MyDialog implements ActionListener, ItemListener {
    private Panel pEingabe = new Panel ();
    private Label errMsg = new Label ("", Label.CENTER);
    private TextField tfNr = new TextField("0", 1);
    private TextField tfKassa = new TextField("0", 1);    
    private Button bCancel = new Button ("Cancel");
    private Button bOk = new Button ("OK");
    private Daten daten;
    private Liste rechnungsdaten;

    
    /**
     * Konstrukor.
     * Hier wird der Dialog initialisiert.
     */
    DialogNeu (Frame parent, String titel, boolean modal, Daten d) {
	super(parent,titel);
	daten = d;
	rechnungsdaten = d.rechnungsdaten;
	setBackground(Color.lightGray);
	setResizable(false);
	setLayout(new BorderLayout());
	
	pEingabe.setLayout (new GridLayout(3,2));
	pEingabe.add (new Label ("Rechnungnummer:"));
	pEingabe.add (new Label ("Kassa:"));
	pEingabe.add (tfNr);
	pEingabe.add (tfKassa);
	pEingabe.add (bOk);
	pEingabe.add (bCancel);
	
	bOk.addActionListener (this);
	bCancel.addActionListener (this);
	
	add ("Center", pEingabe);
	add ("South", errMsg);

	// WindowListener zum Schlieen des Dialoges
	addWindowListener(new Dialogschliesser (this));
	pack();
	}
	
    
    /**
     * Beendet den Dialog und kehrt zurck zum Hauptframe.
     */
    public void cancel() {
	setVisible(false);
	dispose();
	((Window)getParent()).toFront();
	getParent().requestFocus();
	}


    /**
     * Wenn Button gedrckt wurde	    	    
     */
    public void actionPerformed (ActionEvent e) {		    
	// Beendet Bearbeitung
	if (e.getActionCommand().equals("Cancel")) cancel();
	
	// Fgt neue Rechnung hinzu
	else if (e.getActionCommand().equals("OK")) add();
	}
	
    
    /**
     * methode itemStateChanged.
     * Verarbeitet die Item Events des Choice- und Listobjekts..
     */
    public void itemStateChanged (ItemEvent e) {
	}

		
    /**
     * Fgt die neue Rechnung zu den Daten hinzu.
     */
    public void add() {
	String sElement;
	int nr = 0, kassa = 0;
	boolean err = false;

	errMsg.setText ("");		
	
	try {nr = Integer.parseInt (tfNr.getText ());}
	catch (
	    NumberFormatException e) {errMsg.setText ("Ungltige Rechnungsnummer");
	    err = true;
	    }
	    
	try {kassa = Integer.parseInt (tfKassa.getText ());}
	catch (
	    NumberFormatException e) {errMsg.setText ("Ungltige Kassennummer");
	    err = true;
	    }
	
	// Wenn die Rechnungsnummer ungltig ist
	if ((nr <= 0) && !err) {
	    err = true;
	    errMsg.setText ("Rechnungsnummer zu klein");
	    }
	
	// Wenn die Kassennummer ungltig ist
	else if ((kassa <= 0) && !err) {
	    err = true;
	    errMsg.setText ("Kassennummer zu klein");
	    }
	
	// Wenn die Rechnung schon existiert    
	for (rechnungsdaten.gotoStart (); !rechnungsdaten.eol (); rechnungsdaten.gotoNext ())
	    if (((Rechnung) rechnungsdaten.getData()).rechnungsNr == nr) {
		err = true;
		errMsg.setText ("Rechnung existiert bereits");
		}

	// Wenn kein Fehler aufgetreten ist, wird die neue Rechnung angelegt
	if (!err) {
	    Liste l = new Liste ();
	    
	    daten.addRechnung (nr, kassa, l);
	    errMsg.setText ("Daten angelegt");
	    }
    	}
    }    
    
    
/**
 * Klasse DialogNeuesElement.
 * Ermglicht das Einfgen eines neuen Rechnungselements.
 * Dies geschieht ber einen modalen Dialog.
 */
class DialogNeuesElement extends MyDialog implements ActionListener {
    Button bOk = new Button ("OK");
    Button bCancel = new Button ("Cancel");
    Panel p = new Panel ();
    TextField tf = new TextField ("1", 1);
    Choice choiceArtikel = new Choice ();
    Liste artikelDaten;
    Liste rechnungsDaten;
    Liste rechnungsElemente;
    Daten daten;
    String nr;
    

    /**
     * Kostruktor.
     * ffnet den modalen Dialog.
     * @param parent Referenz auf das Mutterfenster.
     * @param n die Rechnung die um einen Artikel erweitert werden soll.
     * @param d die Daten.
     */
    DialogNeuesElement (Frame parent, String n, Daten d) {
	super (parent, "Artikel hinzufgen", true);
	artikelDaten = d.artikeldaten;
	rechnungsDaten = d.rechnungsdaten;
	daten = d;
	nr = n;

	// Setzt Parameter fr Dialog	
        setBackground(Color.lightGray);
        setResizable(false);
        setLayout(new BorderLayout());
	
	// Sucht Rechnung
	for (rechnungsDaten.gotoStart (); !rechnungsDaten.eol (); rechnungsDaten.gotoNext ())
	    if (((Rechnung) rechnungsDaten.getData()).rechnungsNr == Integer.parseInt (nr))
		    rechnungsElemente = ((Rechnung) rechnungsDaten.getData()).elements;
			
	// Liste fllen, wobei schon in der Rechnung enthaltene Artikel nicht aufgelistet werden
        for (artikelDaten.gotoStart (); !artikelDaten.eol (); artikelDaten.gotoNext ()) {
	    boolean fund = false;
	    for (rechnungsElemente.gotoStart (); !rechnungsElemente.eol (); rechnungsElemente.gotoNext ())
		if (((RechnungsElement) rechnungsElemente.getData ()).artikel == ((Artikel) artikelDaten.getData()).artikel)
		    fund = true;
	    if (!fund) choiceArtikel.add ("" + ((Artikel) artikelDaten.getData()).name);
	    }
	
	bOk.addActionListener (this);
	bCancel.addActionListener (this);
	tf.setText ("1");
	
	// Fgt buttons in Panel ein
	p.setLayout (new GridLayout(2, 2));
	  p.add (new Label ("Menge:"));
	  p.add (tf);
	  p.add (bOk);
	  p.add (bCancel);
	
	// Positioniert die Objekte im Dialog
	add ("North", choiceArtikel);
	add ("Center", p);	

        // WindowListener zum Schlieen des Dialoges
        addWindowListener(new Dialogschliesser (this));
	
	// Richtet Objekte ein
        pack();
        }


    /**
     * Methode cancel ().
     * Beendet den Dialog und kehrt zurck zum Mutterfenster..
     */
    public void cancel() {
	setVisible (false);
	dispose();
	((Window) getParent()).toFront();
	getParent().requestFocus();
	}		        	    	    


    /**
     * Verarbeitet ActionEvents.
     * @param e das ausgelte Event.
     */
    public void actionPerformed (ActionEvent e) {		    
	// Beendet Bearbeitung, wenn Eingabe korrekt
	if (e.getActionCommand().equals ("OK")) 
	    {
	    boolean err = false;
	    int menge = 0;
	    try {menge = Integer.parseInt (tf.getText ());}
	    catch (NumberFormatException ex) {err = true;}
	    
	    // Wenn kein Fehler aufgetreten ist - sichern
	    if ((menge > 0) && !err) {
		// selektierten Artikel auslesen und Artikelnummer bestimmen
		String artikel = choiceArtikel.getSelectedItem();
		for (artikelDaten.gotoStart (); !artikelDaten.eol (); artikelDaten.gotoNext ())
		    if (artikel.equals (((Artikel) artikelDaten.getData ()).name))
			daten.addrechnungsElement (rechnungsElemente, ((Artikel) artikelDaten.getData ()).artikel, menge);

		// den Dialog schliessen	    
		cancel ();	  
		}
	    }
	    
	// Beendet Bearbeitung sofort - ohne nderung
	if (e.getActionCommand().equals ("Cancel")) 
	    cancel ();
	}        
    }
    