/**
 * Klasse: Liste.
 * Speichert und Verwaltet dynamische Datenstrukturen in einer doppelt verketteten Liste.
 */
public class Liste {
    // Hilfsnodes, zur Verwaltung und Ansteuerung der Liste
    public Node start, now, end;

    /**
     * Konstruktor.
     * Hier werden die Hilfsnodes zum Verwalten der Daten initialisiert.
     */    
    public Liste () {
	now = null;
	start = null;
	end = null;
	}
    
    /**
     * Methode: add.
     * Fgt ein neues Element in die Liste mit der gre des bergebenen Objektes ein.
     * Falls kein aktuelles Element ausgewhlt (now = null), wird das Element am Ende der
     * Liste angefgt.
     * @param x - das Objekt, das in die Liste eingefgt werden soll.
     */
    public void add (Object x) {
    
	// Wenn kein Element initialisiert ist: neues Element erzeugen und Hilfsnodes setzen
	if (start == null)
	    {
	    start = new Node (x, null, null);
	    end = now = start;
	    }
	// Wenn schon mind. ein Element initialisiert ist: neues Element erzeugen
	else
	    {
	    if (now == null) now = end;
	    Node T = new Node (x, now.next, now);
	    now.next = T;
	    now = T;
	    if (now.next == null)
		end = T;
	    }
	}
    
    /**
     * Methode: eol.
     * berprft, ob das Ende der Liste erreicht wurde.
     * @return true, wenn das Ende der Liste erreicht, sonst false.
     */
    public boolean eol () {
	return now == null;
	}
    
    /**
     * Methode: gotoNext.
     * Setzt den Hilfspointer now auf das nchste Element in der Liste.
     */
    public void gotoNext () {
	if (now != null) now = now.next;
	}
    
    /**
     * Methode: gotoLast.
     * Setzt den Hilfspointer now auf das vorherige Element in der Liste.
     */
    public void gotoLast () {
	if (now != null) now = now.last;
	}
    
    /**
     * Methode: gotoStart.
     * Setzt den Hilfspointer now zurck auf den Start der Liste.
     */
    public void gotoStart () {
	now = start;
	}
    
    /**
     * Methode: getData.
     * @return Liefert das gespeicherte Objekt, der aktuellen Listenposition zurck.
     */
    public Object getData () {
	return now.data;
	}

		
    /**
     * Methode: destroy.
     * Setzt alle Hilfnodes auf null zurck -> Lschen der Liste.
     */
    public void destroy () {
	now = end = start = null;
	}
	
	
    /**
     * Methode: remove ().
     * Entfernt das aktuelle Element aus der Liste.
     */
    public void remove () {
	// Wenn das erste Listenelement: lschen
	if ((now == start) && (now != end)) {	    
	    now = start = now.next;
	    now.last = null;
	    }
	    
	// Wenn das letzte Listenelement: lschen
	else if ((now == end) && (now != start)) {
	    end = now = now.last;
	    now.next = null;
	    }

	// Wenn Element in Mitte: lschen
	else if ((now != end) && (now != start)) {
	    Node T = now.last;
	    Node S = now.next;
	    T.next = now.next;
	    S.last = now.last;
	    now = T;
	    }
			
	// Wenn nur ein Element in der Liste enthalten: Liste lschen
	else if (start == end)
	    destroy ();
	}

        
    /**
     * berprft, ob die Liste leer ist.
     * @return True, falls die Liste leer ist.
     */
    public boolean isEmpty () {
	if (start == null)
	    return true;
	return false;
	}


    /**
     * Bestimmt die Anzahl der Eintrge.
     * @return Anzahl der Eintrge.
     */
    public int length () {
	int anz = 0;
	if (isEmpty ()) return anz;
	for (gotoStart (); !eol (); gotoNext ())
	    anz ++;
	return anz;
	}
    }
    

	
        
/**
 * Klasse: Node.
 * In diesem Format werden die einzelnen Elemente der Liste abgelegt.
 */
class Node {
    /**
     * Beinhaltet die Abgelegten Daten.
     */
    Object data;
    
    /**
     * Zeigt auf den nchsten Listeneintrag.
     */
    Node next;
    
    /**
     * Zeigt auf den vorherigen Listeneintrag.
     */
    Node last;
	
    
    /**
     * Konstruktor Node.
     * Erzeugt ein neues Listenelement.
     * @param d das Objekt (Datenelement) das eingefgt werden soll.
     * @param n der Verweis auf das nchste Element.
     * @param l der Verweis auf das vorhergehende Element.
     */
    Node (Object d, Node n, Node l)
        {
        data = d;
        next = n;
        last = l;
        }
    }
