import java.awt.*;
import java.awt.event.*;
import java.util.*;

/**
 * class MyGUI.
 * Hier wird die graphische Ausgabe erzeugt.
 * Mit Hilfe des Observer Interface werden die Rechnungsfiles berwacht.
 */
public class MyGUI extends Frame implements ItemListener, Observer, ActionListener {

    private Daten daten;
    private CheckboxGroup cbg1, cbg2;
    private Checkbox artikel, kassa, balken, kreis;
    private ScrollPane scrollpane;
    private OutputCanvas output;
    private Label header;
    
    private final static int BL_VGAP = 10;
    private final static int BL_HGAP = 10;
    
    /**
     * Konstruktor.
     * ffnet Frame und initialisiert die Anzeige.
     * @param artikelDaten - die Artikeldaten.
     * @param rechnungsDaten - die Rechnungsdaten.
     * @see Artikel
     * @see Rechnung
     * @see OutputCanvas
     * @see Fensterschliesser
     */
    MyGUI (Daten daten) {
	// Initialisiert Frame
    	super ("Rechnungsbearbeitung");
	setSize (600, 400);
	setBackground (Color.lightGray);
	addWindowListener(new Fensterschliesser ());
	Menue mb = new Menue (this);
	setMenuBar (mb);

	// kopiert die Referenzen
	this.daten = daten;

	// Setzt Leyout auf BorderLayout
	setLayout (new BorderLayout(BL_HGAP, BL_VGAP));
	
	  // Erzeugt Label:
	  header = new Label ("", Label.CENTER);
	  add ("North", header);
	  
	  // Erzeugt den Canvas + Scrollpane
	  scrollpane = new ScrollPane ();
	  output = new OutputCanvas (scrollpane, header, daten);
	  scrollpane.add (output);
	  add ("Center", scrollpane);
	  scrollpane.doLayout ();

	  // Setzt Scrollbareinheiten
	  scrollpane.getHAdjustable().setUnitIncrement (40);
	  scrollpane.getVAdjustable().setUnitIncrement (30);	
	
	  // Erzeugt die Radiobuttons und deren Listener
	  Panel p = new Panel ();
	    p.setLayout (new GridLayout(3, 2));
	    cbg1 = new CheckboxGroup();
	    cbg2 = new CheckboxGroup();
	    p.add (new Label ("Was anzeigen?", Label.LEFT));
	    p.add (new Label ("Wie anzeigen?", Label.LEFT));
	    artikel = new Checkbox("Artikel", cbg1, true);
	    artikel.addItemListener (this);
	    p.add (artikel);
	    balken = new Checkbox("Balkendiagramm", cbg2, true);
	    balken.addItemListener (this);
	    p.add (balken);
	    kassa = new Checkbox("Kassen", cbg1, false);
	    kassa.addItemListener (this);
	    p.add (kassa);
	    kreis = new Checkbox("Kreisdiagramm", cbg2, false);
	    kreis.addItemListener (this);
	    p.add (kreis);
	  add ("South", p);

	// Zeigt fertigen Frame an	
	setVisible (true);
	}
    
    
    /**
     * Methode: itemStateChanged.
     * Wertet Vernderungen in der CheckBoxGroup aus & veranlasst ein Neuzeichnen des Canvas.
     */
    public void itemStateChanged (ItemEvent e) {
        if ((e.getSource() == kassa) && output.artikel)
		output.artikel = false;
	else if ((e.getSource () == artikel) && !output.artikel)
		output.artikel = true;
        else if ((e.getSource () == kreis) && output.balken)
		output.balken = false;
	else if ((e.getSource () == balken) && !output.balken)
		output.balken = true;
	
	output.repaint ();
	}
	

    /**
     * Wertet die ActionEvents des Mens aus.
     */
    public void actionPerformed (ActionEvent e) {
	String s = e.getActionCommand();
	
	// Dialog zum Rechnung eingeben aufrufen
	if (s.equals("Rechnung Eingeben")) {
	    DialogNeu dNeu = new DialogNeu (this, "Neue Rechnung erstellen", false, daten);
	    dNeu.setVisible(true);	
	    }
	    
	// Dialog zum Rechnung ndern aufrufen
	else if (s.equals("Rechnung ndern")) {
	    DialogBearbeiten dEdit = new DialogBearbeiten (this, "Rechnung Bearbeiten", false, daten);
	    daten.addObserver (dEdit);
	    dEdit.setVisible(true);	
	    }
	    
	// Dialog zum Rechnung lschen aufrufen
	else if (s.equals("Rechnung Lschen")) {
	    DialogLoeschen dLschen = new DialogLoeschen (this, "Rechnung Lschen", false, daten);
	    dLschen.setVisible(true);	
	    }
	    
	// Dialog zum klonen des Fensters
	else if (s.equals("Diagramm klonen")) {
	    // Startet graphische Ausgabe
	    MyGUI m = new MyGUI (daten);
	
	    // Initialisiert Observer
	    daten.addObserver (m);	    
	    }
	    
	// Programm beenden
	else if (s.equals("Beenden")) System.exit(0);
	}	


    /**
     * Methode: Update.
     * Fhrt bei einer Vernderung des zu berwachenden Objektes die Repaintmethode des Canvas aus.
     * @param o - das berwachte Objekt.
     * @param arg - die Vernderung.
     */
    public void update (Observable o, Object arg) {
	output.repaint ();
	}
    }


/**
 * Klasse: OutputCanvas.
 * Die Zeichenflche. Die Abmessungen des Canvas werden durch die auszugebenden Strings definiert.
 * Der Canvas ist aber mindestens so gro, wie der geffnete Frame.
 */
class OutputCanvas extends Canvas {
    public boolean artikel = true, balken = true;
	
    private ScrollPane scrollpane;
    private Label header;
    private Diagramm diagramm;
    private Daten daten;
    
    // Die Ausgabetexte fr die berschrift
    private final static String LBL_1 = "Verkaufte Artikel als Balkendiagramm";
    private final static String LBL_2 = "Verkaufte Artikelgruppen als Kreisdiagramm";
    private final static String LBL_3 = "Verkaufte Artikel pro Kassa als Balkendiagramm";
    private final static String LBL_4 = "Verkaufte Artikel pro Kassa als Kreisdiagramm";

    /**
     * Konstruktor.
     * Hier wird die Gre des Canvas berechnet & gesetzt.
     * @param scrollpane der ScrollPane, in dem der Canvas beinhaltet ist.
     * @param header das Label in dem die berschrift angezeigt werden soll.
     * @param daten das Datenobjekt.
     */
    OutputCanvas (ScrollPane scrollpane, Label header, Daten daten) {
        this.scrollpane = scrollpane;
	this.header = header;
        this.daten = daten;
        setBackground (Color.white);
        Dimension groesse = scrollpane.getViewportSize ();
        setSize (groesse.width, groesse.height);
        }


    /**
     * Methode: paint.
     * Hier wird das gewnschte Diagramm ausgegeben.
     **/
    public void paint (Graphics g) {
	
	// Auswertung, welches Diagramm gezeichnet werden soll
	// Balkendiagramm der gekauften Artikel
	if (artikel && balken) {
	    header.setText (LBL_1);
	    diagramm = new Balkendiagramm (g, new ArtikelDaten (daten), scrollpane, this);
	    }
	    
	// Kreisdiagramm der gekauften Artikelgruppen
	else if (artikel && !balken) {
	    header.setText (LBL_2);
	    diagramm = new Kreisdiagramm (g, new GruppenDaten (daten), scrollpane, this);
	    }
	    
	// Balkendiagramm der gekauften Artikel pro Kassa
	else if (!artikel && balken) {
	    header.setText (LBL_3);
	    diagramm = new Balkendiagramm (g, new KassaDaten (daten), scrollpane, this);
	    }
		
	// Balkendiagramm der gekauften Artikel pro Kassa
	else if (!artikel && !balken) {
	    header.setText (LBL_4);
	    diagramm = new Kreisdiagramm (g, new KassaDaten (daten), scrollpane, this);
	    }
        }
    }


/**
 * Klasse: Fensterschliesser.
 * Implementiert das windowClosing Event zum Schlieen des Frames.
 */
class Fensterschliesser extends WindowAdapter {
    public void windowClosing (WindowEvent e) {
	System.exit (0);}
    }	
    